var _ = require('lodash');

var AWS = require('./AWS').AWS;
var AWSCognito = require('./AWS').AWSCognito;


const _signUp = function(username, password, attributes) {
    let self = this

    return new Promise((resolve, reject) => { 
        if (_.isNil(self.awsPoolInfo)) {
            const err = {
                message: 'Missing AWS Pool Info'
            }
            reject(err);
            return;
        }

        const userPool = new AWSCognito.CognitoUserPool(self.awsPoolInfo);
        
        let attributeList = [];

        if (!_.isEmpty(attributes)) {
            attributeList = Object.keys(attributes).map(key => {
                return new AWSCognito.CognitoUserAttribute({ Name: key, Value: attributes[key] });
            });      
        }
        
        const authDetails = self.makeAuthDetails(username, password);

        userPool.signUp(
            username,
            password,
            attributeList,
            null,
            (err, result) => {
                if (err) {
                    reject(err);
                    return;
                }
                
                const cognitoUser = result.user;

                resolve({ cognitoUser, authDetails });

            }
        );
    })
}

const signUp = function(username, password, attributes, autoConfirm) {
    let self = this;

    if (autoConfirm) {
        return self._signUp(username, password, attributes)
            .then(self.authenticate.bind(self));
    }
    else {
        return self._signUp(username, password, attributes)
    }
}

const authenticate = (params) => {
    
    let cognitoUser = params.cognitoUser;
    let authDetails = params.authDetails;

    return new Promise((resolve, reject) => {
        cognitoUser.authenticateUser(authDetails, {
            onSuccess: (session) => {
                resolve({ cognitoUser, session });
            },

            onFailure: (err) => {
                reject(err);
            }
        })
    })
}

const makeAuthDetails = function (username, password) {
    const authData = {
        Username: username,
        Password: password
    };

    const authDetails = new AWSCognito.AuthenticationDetails(authData);
    return authDetails;
}

const makeUser = function (params) {
    let email = params.email;

    const userPool = new AWSCognito.CognitoUserPool(this.awsPoolInfo);

    const userData = {
        Username: email,
        Pool: userPool
    }

    const cognitoUser = new AWSCognito.CognitoUser(userData);

    return cognitoUser;
}

const signIn = function (params) {
    let email = params.email;
    let password = params.password;
    let self = this;
    const first = new Promise((resolve, reject) => {
        
        const authDetails = self.makeAuthDetails(email, password);        
        const cognitoUser = self.makeUser({ email, password });
        
        resolve({cognitoUser, authDetails});
    });

    return first.then(self.authenticate.bind(self));

}

const signOut = function (cognitoUser, globally) {
    if (_.isNil(cognitoUser)) {
        throw new Error('No user to sign out');
    }

    if (globally) {
        cognitoUser.globalSignOut();
    }
    else {
        cognitoUser.signOut();
    }
}

const deleteAccount = function (cognitoUser) {
    let self = this;
    return new Promise((resolve, reject) => { 
        if (_.isNil(cognitoUser)) {                        
            reject(new Error('User is not logged in'));
            return;
        }

        cognitoUser.deleteUser((err, result) => { 
            if (err) {
                console.log(err.message ? err.message : 'Error deleting user');                    
                reject(err);
                return;
            }

            resolve();
        });
    });
}

const storageSync = function () {
    const userPool = new AWSCognito.CognitoUserPool(this.awsPoolInfo);
    return new Promise((resolve, reject) => {
        userPool.storage.sync((err, result) => {
            if (err) {
                console.log('storage sync failed', err);
                reject(err);
                return;
            }

            resolve(userPool);
        });
    });
}

const retrieveUserAttributes = function (cognitoUser) {
    return new Promise((resolve, reject) => {
        cognitoUser.getUserAttributes((err, result) => { 
            if (err) {
                reject(err);
                return;
            }

            // console.log(result);
            resolve(result);
        });
    });
} 

const getUserSession = function (cognitoUser) {
    return new Promise((resolve, reject) => {
        console.log('getting session');
        const user = cognitoUser;
        cognitoUser.getSession((err, session) => {
            if (err) {
                reject(err);
                return;
            }

            if (!session.isValid()) {
                reject(new Error('Invalid user session'));
                return;
            }
            resolve({ user, session });
        });
    });
}

const getCurrentUser = function () {
    let self = this;
    return self.storageSync()
        .then((userPool) => {
            const cognitoUser = userPool.getCurrentUser();
            
            if (_.isNil(cognitoUser)) {
                return Promise.reject(new Error('No user currently logged in'));
            }

            return self.getUserSession(cognitoUser)
        });
}

const verifyEmailForUser = function (cognitoUser, pinCode) {
    return new Promise((resolve, reject) => {
        if (_.isNil(cognitoUser)) {
            reject(new Error('Missing cognito user for email verification'));
            return;
        }         
        
        if (_.isNil(pinCode)) {
            reject(new Error('Missing verification code for email verification'));
            return;
        }    

        console.log('verifying cognito user \'email\'');
        cognitoUser.getAttributeVerificationCode('email', {
            onSuccess: (result) => {
                console.log('verifying cognito user success');                    
                resolve(result);
            },
            onFailure: (err) => {
                console.log(`verifying cognito user failed ${err.message}`);                                        
                reject(err);
            },
            inputVerificationCode: () => {
                
                cognitoUser.verifyAttribute('email', pinCode, {
                    onSuccess: (result) => {
                        console.log('verifying attribute email success');                    
                        
                        resolve(result);
                    },
                    onFailure: (err) => {
                        console.log(`verifying attribute email failed ${err.message}`);                    
                        
                        reject(err);
                    },
                });
            }
        });
    });
}

const verifyEmail = function (pinCode) {
    let self = this;
    return new Promise((resolve, reject) => {
        if (_.isNil(self.cognitoUser)) {
            self.getCurrentUser()
                .then((user, session) => {
                    return self.verifyEmailForUser(user, pinCode);
                })
                .then((result) => {
                    console.log('Email verification success');
                    resolve(result);
                })
                .catch((err) => {
                    console.log(err.message);                        
                    reject(err);
                });
        }
    });    
}

const resendVerificationEmail = function (cognitoUser) {
    
    return new Promise((resolve, reject) => {
        if (_.isNil(cognitoUser)) {
            const err = { message: 'Missing cognito user for resending verificaiton email' };
            reject(err);
            return;
        }

        cognitoUser.resendConfirmationCode((err, result) => {
            if (err) {
                reject(err);
                return;
            }

            resolve()
        });
    });
}
    

const confirmRegistration = function (cognitoUser, code, email, password)  {
    return new Promise((resolve, reject) => {
        if (_.isEmpty(cognitoUser)) {
            throw new Error('No user found');
        }
    
        if (_.isEmpty(code)) {
            throw new Error('verification code required');
        }
    
        if (!(/^\d{6}$/.test(code))) {
            throw new Error('6-digit verification code required');        
        }
    
        if (_.isEmpty(email)) {
            throw new Error('missing email');
        }
    
        if (_.isEmpty(password)) {
            throw new Error('missing password');
        }

        cognitoUser.confirmRegistration(code, true, (err, result) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(result);
        });
    });
}
    
const resendConfirmationCode = function (cognitoUser) {
    return new Promise((resolve, reject) => {
        cognitoUser.resendConfirmationCode((err, result) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(result);
        });
    });
}

const forgotPassword = function (cognitoUser) {
    return new Promise((resolve, reject) => { 
        if (_.isNil(cognitoUser)) {
            throw new Error('Missing user');
        }

        cognitoUser.forgotPassword({
            onSuccess: function (result) {
                resolve(result);
            },

            onFailure: function (err) {
                reject(err);
            }
        });
    });
}

const confirmPassword = function (cognitoUser, code, newPassword) {
    return new Promise((resolve, reject) => {
        if (_.isNil(cognitoUser)) {
            throw new Error('Missing user');
        }

        cognitoUser.confirmPassword(code, newPassword,{
            onSuccess: function (result) {
                resolve(result);
            },

            onFailure: function (err) {
                reject(err);
            }
        });
    });
}

const changePassword = function (cognitoUser, oldPassword, newPassword) {
    return new Promise((resolve, reject) => {
        if (_.isNil(cognitoUser)) {
            throw new Error('Missing user when changing password');
        }

        cognitoUser.changePassword(oldPassword, newPassword, function (err, result) {
            if (err) {
                reject(err);
                return;
            }

            resolve(result);
        });
    });
}

function CognitoUserSDK(config) {
    
    if (_.isEmpty(config)) {
        throw new Error('Missing config param');
    }

    this.awsPoolInfo = config;
}


CognitoUserSDK.prototype = {
    signUp: signUp,
    _signUp: _signUp,
    signIn: signIn,
    signOut: signOut,
    authenticate: authenticate,
    makeAuthDetails: makeAuthDetails,
    makeUser: makeUser,
    deleteAccount: deleteAccount,
    forgotPassword: forgotPassword,
    confirmPassword: confirmPassword,
    changePassword: changePassword,
    getCurrentUser: getCurrentUser,
    getUserSession: getUserSession,
    retrieveUserAttributes: retrieveUserAttributes,
    confirmRegistration: confirmRegistration,
    resendConfirmationCode: resendConfirmationCode,
    storageSync: storageSync
}


module.exports = CognitoUserSDK;
